#!/usr/bin/env python
# -*- coding: UTF-8 -*-
###########################################################################
# Eole NG - 2007
# Copyright Pole de Competence Eole  (Ministere Education - Academie Dijon)
# Licence CeCill  cf /root/LicenceEole.txt
# eole@ac-dijon.fr
#
# get_date_maj.py
#
# script de vérification de la dernière date de mise à jour
#
###########################################################################

import sys, optparse, os, urllib, socket
from zephir.backend.config import PATH_ZEPHIR
from zephir.config import DISTRIBS
from ConfigParser import ConfigParser
from creole.parsedico import parse_dico
import apt_pkg

def check_maj_md5(serveur_maj, version="2.2", codename="hardy", zephir_version=4):
    """vérifie la présence de mises à jour"""
    DISTRIB_CODENAME = codename

    # gestion des différentes arborescences pour les dépots eole
    if zephir_version < 4:
        eole_dirs = ['']
    else:
        eole_dirs = ['i386','all']

    repositories = [('eoleng/eole-%s' % version, eole_dirs, True),
                    ('ubuntu/dists/%s' % DISTRIB_CODENAME, ['binary-i386','main/binary-i386','universe/binary-i386','restricted/binary-i386'], True),
                    ('eoleng/eole-%s-security' % version, eole_dirs, True),
                    ('ubuntu/dists/%s-security' % DISTRIB_CODENAME, ['binary-i386','main/binary-i386','universe/binary-i386','restricted/binary-i386'], True),
                    ('eoleng/eole-%s-updates' % version, eole_dirs, False),
                    ('ubuntu/dists/%s-updates' % DISTRIB_CODENAME, ['binary-i386','main/binary-i386','universe/binary-i386','restricted/binary-i386'], False)]

    c=ConfigParser()
    if os.path.isfile(os.path.join(PATH_ZEPHIR, 'md5maj_%s.ini' % DISTRIB_CODENAME)):
        c.read(os.path.join(PATH_ZEPHIR, 'md5maj_%s.ini' % DISTRIB_CODENAME))
    updates = []
    for serv_maj in serveur_maj:
        section_name = 'eole_maj_%s' % serv_maj
        if not c.has_section(section_name):
            c.add_section(section_name)
        # récupération des md5sums
        md5_info = []
        for repository, filepaths, minimum in repositories:
            for arch in filepaths:
                info = parse_release('http://%s/%s/%s/Release' % (serv_maj, repository, arch), '', serv_maj, repository, minimum)
                if info is None:
                    info = parse_release('http://%s/%s/Release' % (serv_maj, repository), arch, serv_maj, repository, minimum)
                if info is not None:
                    md5_info.append(info)
        # traitement des md5
        for repos,md5,minimum,pkg_url in md5_info:
            up_to_date = True
            if c.has_option(section_name, repos):
                # vérification par rapport à l'ancienne valeur
                if c.get(section_name, repos) != md5:
                    up_to_date = False
            else:
                up_to_date = False
            # mise à jour du md5
            c.set(section_name,repos,md5)
            updates.append((repos, up_to_date, minimum, pkg_url))
    return updates, c

def parse_release(f_rel, filepath, serveur_maj, repository, minimum):
    url=urllib.urlopen(f_rel)
    data = url.read().split('SHA1:\n')[0]
    # for filepath in filepaths:
    for line in data.split('\n'):
        line = line.strip().split()
        if len(line) > 0:
            # md5 du fichier Packages
            if line[-1] == os.path.join(filepath, 'Packages.gz'):
                pkg_url='%s/%s' % (os.path.dirname(f_rel), os.path.join(filepath, 'Packages.gz'))
                return (os.path.join(repository,filepath), line[0], minimum, pkg_url)
    return None

def maj_listes(updates, serveur_maj, codename="hardy"):
    """construit une liste de paquets par repository"""
    apt_pkg.InitSystem()
    for serv_maj in serveur_maj:
        c = ConfigParser()
        if os.path.isfile(os.path.join(PATH_ZEPHIR, 'packages_%s_%s.ini' % (codename, serv_maj))):
            c.read(os.path.join(PATH_ZEPHIR, 'packages_%s_%s.ini' % (codename, serv_maj)))
        for repos, state, minimum, pkg_url in updates:
            if state == False:
                # lecture et traitement du fichier 'Packages'
                pkg_name = ""
                pack_temp = os.path.join(PATH_ZEPHIR, 'Packages')
                urllib.urlretrieve(pkg_url, "%s.gz" % pack_temp)
                res = os.system("""/bin/gunzip -c %s.gz | grep -E "^(Package:|Version:)" > %s""" % (pack_temp, pack_temp))
                if os.path.isfile(pack_temp):
                    info_pkgs = file(pack_temp)
                    for line in info_pkgs:
                        if line.startswith("Package:"):
                            pkg_name = line.split()[1].strip()
                        if line.startswith("Version:"):
                            pkg_version = line.split()[1].strip()
                            # stockage des infos
                            if not c.has_section(pkg_name):
                                c.add_section(pkg_name)
                            if minimum == True:
                                # paquets pour maj minimum
                                try:
                                    old_rev=c.get(pkg_name, 'minimum')
                                    assert apt_pkg.VersionCompare(pkg_version, old_rev) <= 0
                                except:
                                    # paquet plus récent ou pas de paquet repéré avant, on update
                                    c.set(pkg_name, 'minimum', pkg_version)
                            # paquets pour maj_complete
                            try:
                                old_rev=c.get(pkg_name, 'complete')
                                assert apt_pkg.VersionCompare(pkg_version, old_rev) <= 0
                            except:
                                # paquet plus récent ou pas de paquet repéré avant
                                c.set(pkg_name, 'complete', pkg_version)
                                # si le paquet de la maj minimum est plus récent on le prend
                                try:
                                    min_rev = c.get(pkg_name, 'minimum')
                                    if apt_pkg.VersionCompare(pkg_version, min_rev) <= 0:
                                        c.set(pkg_name, 'complete', min_rev)
                                except:
                                    pass
                    os.unlink(pack_temp)
                    os.unlink("%s.gz" % pack_temp)
                info_pkgs.close()
        f_pkg_ini = open(os.path.join(PATH_ZEPHIR, 'packages_%s_%s.ini' % (codename, serv_maj)), 'w')
        c.write(f_pkg_ini)
        f_pkg_ini.close()

if __name__ == '__main__':
    socket.setdefaulttimeout(5)
    dico = parse_dico()
    # test de connexion au serveur de maj
    for serv_maj in dico['serveur_maj_clients']:
        try:
            url=urllib.urlopen('http://%s/eoleng/' % serv_maj)
        except IOError:
            # téléchargement des informations impossibles
            sys.exit("Erreur d'accès au serveur de mise à jour %s" % serv_maj)
    # mise à jour des différentes distributions gérées
    for zephir_version, dist_infos in DISTRIBS.items():
        if zephir_version > 1:
            codename, version, maintained = dist_infos
            updates, md5_ini = check_maj_md5(dico['serveur_maj_clients'], version, codename, zephir_version)
            # si il y a un changement, on lance Query-Auto et on met à jour les infos
            # sur les paquets disponibles
            for repos, state, minimum, pkg_url in updates:
                if state == False:
                    # mise à jour des infos
                    maj_listes(updates, dico['serveur_maj_clients'], codename)
                    # sauvegarde des md5 pour la prochaine vérification
                    f_maj = open(os.path.join(PATH_ZEPHIR, 'md5maj_%s.ini' % codename),'w')
                    md5_ini.write(f_maj)
                    f_maj.close()
                    # listes mises à jour, on arrête la boucle
                    break
    sys.exit(0)
