#!/bin/bash
###########################################################################
# Eole NG - 2007
# Copyright Pole de Competence Eole  (Ministere Education - Academie Dijon)
# Licence CeCill  cf /root/LicenceEole.txt
# eole@ac-dijon.fr
#
# migration.sh
#
# outil de migration des données d'une serveur Zephir-1.0 vers Zephir NG
# bruno boiget <bruno.boiget@ac-dijon.fr>
#
###########################################################################

# color ASCII codes
#
DEFAULT="\e[0m"
BOLD="\e[1m"
BLACK="\e[30m"
RED="\e[31m"
GREEN="\e[32m"
YELLOW="\e[33m"
BLUE="\e[34m"
VIOLET="\e[35m"
CYAN="\e[36m"
WHITE="\e[37m"


# global vars
Lock="sauvegarde"
Excl="maj"
RepEole=${RepEole="/usr/share/eole"}
. ParseDico
pg_version=8.4
backup_dir=/var/lib/zephir_backups
pg_data=/var/lib/postgresql/$pg_version/main
pg_dir=/etc/postgresql/$pg_version/main
pg_perms=$pg_dir/pg_hba.conf
pg_service=postgresql-$pg_version
initdb=/usr/lib/postgresql/$pg_version/bin/initdb

# récupération mot de passe et archive à restaurer
pass_zeph="toto"
pass_ok=0
stty -echo
while [ $pass_ok -ne 1 ]
do
	read -p "Mot de passe de l'administrateur $admin_zephir (rien pour sortir) ? : " pass_zeph
	[ -z "$pass_zeph" ] && exit 0
	# test de bind ldap
	DN=`ldapsearch -x -h $adresse_ip_ldap -b "$ldap_base_dn" "uid=$admin_zephir" | grep dn | cut -d" " -f2` &>/dev/null
	ldapsearch -x -h $adresse_ip_ldap -b "$ldap_base_dn" -D "$DN" -w "$pass_zeph" &> /dev/null
	if [ $? -eq 0 ]
	then
		pass_ok=1
	else
		echo "Erreur d\'authentification LDAP"
	fi
done
stty echo

if [ ! -d $backup_dir ]
then
	echo -e "${RED} Erreur, répertoire des sauvegardes non trouvé ${DEFAULT}"
	exit 1
fi
pushd $backup_dir &>/dev/null 2>&1

if [ ! -f $1 ]
then
	archive=$1
        basename=`echo -e $archive | sed -e s/"\.tar\.gz"/""/`
else
	archive=/tmp
fi

while [ ! -f $archive ]
do
	# pas de fichier passé en paramètre
	save_files=`ls backup_*.tar.gz | sed -e s/".tar.gz"/""/`
	echo -e "     Utilitaire de migration zephir"
	echo -e
	echo -e "${RED}!!Attention : toutes les données actuelles seront écrasées!!${DEFAULT}"
	echo -e
	echo -e "${CYAN}liste des archives présentes :${DEFAULT}"
	echo -e
	echo -e "$save_files"
	echo -e
	read -p "sauvegarde à restaurer (rien pour sortir): " basename
	if [ -z $basename ]
	then
		exit 0
	fi
	if [ -f $basename.tar.gz ]
	then
		archive=$basename.tar.gz
	else
		echo -e "fichier invalide : $basename.tar.gz"
	fi
done

# décompression et vérification de l'archive
echo -e "décompression en cours..."
tar xzf $archive
cd $basename
echo -e "vérification des données..."
if [ ! -f backup_db.sql ]
then
	echo -e "${RED}sauvegarde de la base de données non trouvée${DEFAULT}"
	exit 1
fi
if [ ! -f ldap.ldif ]
then
	echo -e "${RED}sauvegarde de la base ldap non trouvée${DEFAULT}"
	echo -e "${CYAN}la sauvegarde doit être au format ldif${DEFAULT}"
        echo -e "${CYAN}utilisez l'outil de migration sur la machine d'origine et non la sauvegarde standard${DEFAULT}"
	exit 1
fi
if [ ! -f var_lib_zephir.tar ]
then
	echo -e "${RED}sauvegarde des configurations des clients non trouvée${DEFAULT}"
	exit 1
fi
if [ ! -f etc_uucp.tar ]
then
	echo -e "${RED}sauvegarde des configurations uucp non trouvée${DEFAULT}"
	exit 1
fi
if [ ! -f var_spool_uucp.tar ]
then
	echo -e "${RED}sauvegarde des clés de connexion des clients non trouvée${DEFAULT}"
	exit 1
fi
if [ ! -f etc_eole.tar ]
then
	echo -e "${RED}sauvegarde de la configuration du serveur non trouvée${DEFAULT}"
	exit 1
fi

rep="N"
# on arrête le service zephir pour stopper l'activité sur la base
service zephir stop > /dev/null 2>&1
echo -e "restauration des données..."
# restauration des données de la base postgresql
echo -e " - base postgresql"
cp -f $pg_perms /tmp/pg_hba.savezephir
cp -f $pg_dir/postgresql.conf /tmp/postgresql.conf
service $pg_service stop &> /dev/null
rm -rf $pg_data/*
echo -e "   initialisation de la base"
su - postgres -c "$initdb -D $pg_data/" &> /dev/null
# postgres : se mettre en mode sans mot de passe
echo -e "local\tall\tall\ttrust" > $pg_perms
chown postgres.postgres $pg_perms
chmod 600 $pg_perms
mv -f /tmp/postgresql.conf $pg_dir/postgresql.conf
chown postgres.postgres $pg_dir/postgresql.conf
chmod 600 $pg_dir/postgresql.conf
service $pg_service start &> /dev/null
echo -e "   conversion au format utf-8"
iconv -f iso-8859-1 -t utf-8 backup_db.sql > backup_utf.sql
sleep 2
echo -e "   injection des données"
su postgres -c "psql template1 -f backup_utf.sql" &> /dev/null
# restauration des répertoires et fichiers importants
echo -e " - configuration des serveurs"
# /var/lib/zephir/
rm -rf /var/lib/zephir
tar -C /var/lib/ -xpf var_lib_zephir.tar >/dev/null 2>&1
# /etc/uucp/
rm -rf /etc/uucp
tar -C /etc/ -xpf etc_uucp.tar >/dev/null 2>&1
# /var/spool/uucp/
rm -rf /var/spool/uucp
tar -C /var/spool -xpf var_spool_uucp.tar >/dev/null 2>&1
echo -e " - configuration eole"
# rm -rf /etc/eole/dicos /etc/eole/patch /etc/eole/config.eol
tar -xf etc_eole.tar >/dev/null 2>&1
# recréation du mot de passe
echo -e " - regénération du mot de passe"
/usr/share/eole/gen_pwd.py &> /dev/null
# mise à jour des groupes de droits
echo -e "   mise à jour du schéma de la base"
su postgres -c "psql -d zephir -Uzephir -f /usr/share/zephir/sql/maj.sql" >/dev/null 2>&1
# postgres : se remettre en mode mot de passe
rm -f $pg_perms
mv -f /tmp/pg_hba.savezephir $pg_perms
chown postgres.postgres $pg_perms
chmod 600 $pg_perms
service $pg_service stop &> /dev/null
service $pg_service start &> /dev/null
echo -e " - création des modules Eole NG"
# démarrage du backend zephir sans boucle de vérification des timeouts
service zephir start noscan > /dev/null 2>&1
# mise à jour de la base et création des modules eole2 et reprise du dictionnaire
/usr/share/zephir/utils/migrate_base.py "$pass_zeph"
echo -e " - base ldap"
service slapd stop >/dev/null 2>&1
rm -rf /var/lib/ldap/*
# restauration de la base ldap
slapadd -l ldap.ldif >/dev/null 2>&1
chown -R openldap.openldap /var/lib/ldap
/usr/share/eole/gen_pwd.py --nopgsql > /dev/null 2>&1
service slapd start >/dev/null 2>&1
# on redémarre zephir en mode normal
service zephir restart > /dev/null 2>&1
# purge des fichiers temporaires
echo -e "Supression du répertoire temporaire..."
rm -rf $backup_dir/$basename
echo -e " "
echo -e "${GREEN}Migration terminée"
echo -e "Réinstanciez ou reconfigurez le serveur${DEFAULT}"
echo -e " "
if [ "$activer_client_ldap" == "local" ]
then
	echo -e "si vous avez des problèmes d'authentification avec un utilisateur"
	echo -e "(en particulier admin_zephir), utilisez /usr/share/zephir/utils/add_user.py"
	echo -e "pour mettre à jour son mot de passe dans l'annuaire"
	echo -e " "
fi
## Creer le repertoire .ssh pour uucp
if [ ! -d /var/spool/uucp/.ssh ]
then
	mkdir /var/spool/uucp/.ssh
	chown uucp.uucp /var/spool/uucp/.ssh
	chmod 700 /var/spool/uucp/.ssh
fi
## lancement firewall
/sbin/lance.firewall
# retour au répertoire d'origine
popd >/dev/null 2>&1
